/*---------------------------------------------------------------------------*\

    DAFoam  : Discrete Adjoint with OpenFOAM
    Version : v2

    Description:
        Child class for DASimpleTFoam

\*---------------------------------------------------------------------------*/

#ifndef DAResidualSimpleTFoam_H
#define DAResidualSimpleTFoam_H

#include "DAResidual.H"
#include "addToRunTimeSelectionTable.H"
#include "simpleControl.H"
#include "adjustPhi.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
      Class DAResidualSimpleTFoam Declaration
\*---------------------------------------------------------------------------*/

class DAResidualSimpleTFoam
    : public DAResidual
{
protected:
    /// \name These are state variables, state residuals, and partial derivatives
    //@{
    volVectorField& U_;
    volVectorField URes_;
    volVectorField UResPartDeriv_;

    volScalarField& p_;
    volScalarField pRes_;
    volScalarField pResPartDeriv_;

    volScalarField& T_;
    volScalarField TRes_;
    volScalarField TResPartDeriv_;

    surfaceScalarField& phi_;
    surfaceScalarField phiRes_;
    surfaceScalarField phiResPartDeriv_;
    //@}

    volScalarField& alphat_;

    scalar Pr_ = -9999.0;

    scalar Prt_ = -9999.0;

    /// DATurbulenceModel object
    DATurbulenceModel& daTurb_;

    /// simpleControl object which will be initialized in this class
    simpleControl simple_;

public:
    TypeName("DASimpleTFoam");
    // Constructors

    //- Construct from components
    DAResidualSimpleTFoam(
        const word modelType,
        const fvMesh& mesh,
        const DAOption& daOption,
        const DAModel& daModel,
        const DAIndex& daIndex);

    //- Destructor
    virtual ~DAResidualSimpleTFoam()
    {
    }

    // Members

    /// clear the members
    virtual void clear();

    /// compute residual
    virtual void calcResiduals(const dictionary& options);

    /// update any intermediate variables that are dependent on state variables and are used in calcResiduals
    virtual void updateIntermediateVariables();

    /// update the boundary condition for all the states in the selected solver
    virtual void correctBoundaryConditions();
};

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
